
//
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
// 
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.
//

// This source file was created by a code generator.
// Please do not edit.

#include "qpid/log/Statement.h"
#include "qpid/framing/FieldTable.h"
#include "qpid/management/Manageable.h" 
#include "qpid/management/ManagementAgent.h"
#include "Store.h"


using namespace qmf::com::redhat::rhm::store;
using namespace qpid::framing;
using           qpid::management::ManagementAgent;
using           qpid::management::Manageable;
using           qpid::management::ManagementObject;
using           qpid::management::Args;
using           std::string;

string  Store::packageName  = string ("com.redhat.rhm.store");
string  Store::className    = string ("store");
uint8_t Store::md5Sum[16]   =
    {0x18,0xd,0xd4,0x15,0xd3,0x9a,0xf,0xbe,0x3a,0x40,0xe1,0x1b,0x9e,0x5b,0x7e,0x86};

Store::Store (ManagementAgent*, Manageable* _core, ::qpid::management::Manageable* _parent) :
    ManagementObject(_core)
{
    brokerRef = _parent->GetManagementObject ()->getObjectId ();
    location = "";
    defaultInitialFileCount = 0;
    defaultDataFileSize = 0;
    tplIsInitialized = 0;
    tplDirectory = "";
    tplWritePageSize = 0;
    tplWritePages = 0;
    tplInitialFileCount = 0;
    tplDataFileSize = 0;
    tplCurrentFileCount = 0;
    tplTransactionDepth = 0;
    tplTransactionDepthHigh = 0;
    tplTransactionDepthLow  = 0;
    tplOutstandingAIOs = 0;
    tplOutstandingAIOsHigh = 0;
    tplOutstandingAIOsLow  = 0;



    perThreadStatsArray = new struct PerThreadStats*[maxThreads];
    for (int idx = 0; idx < maxThreads; idx++)
        perThreadStatsArray[idx] = 0;

}

Store::~Store ()
{

    for (int idx = 0; idx < maxThreads; idx++)
        if (perThreadStatsArray[idx] != 0)
            delete perThreadStatsArray[idx];
    delete[] perThreadStatsArray;

}

namespace {
    const string NAME("name");
    const string TYPE("type");
    const string ACCESS("access");
    const string IS_INDEX("index");
    const string IS_OPTIONAL("optional");
    const string UNIT("unit");
    const string MIN("min");
    const string MAX("max");
    const string MAXLEN("maxlen");
    const string DESC("desc");
    const string ARGCOUNT("argCount");
    const string ARGS("args");
    const string DIR("dir");
    const string DEFAULT("default");
}

void Store::registerSelf(ManagementAgent* agent)
{
    agent->registerClass(packageName, className, md5Sum, writeSchema);
}

void Store::writeSchema (Buffer& buf)
{
    FieldTable ft;

    // Schema class header:
    buf.putOctet       (CLASS_KIND_TABLE);
    buf.putShortString (packageName); // Package Name
    buf.putShortString (className);   // Class Name
    buf.putBin128      (md5Sum);      // Schema Hash
    buf.putShort       (11); // Config Element Count
    buf.putShort       (9); // Inst Element Count
    buf.putShort       (0); // Method Count

    // Properties
    ft.clear();
    ft.setString (NAME, "brokerRef");
    ft.setInt    (TYPE, TYPE_REF);
    ft.setInt    (ACCESS, ACCESS_RO);
    ft.setInt    (IS_INDEX, 1);
    ft.setInt    (IS_OPTIONAL, 0);
    buf.put (ft);

    ft.clear();
    ft.setString (NAME, "location");
    ft.setInt    (TYPE, TYPE_SSTR);
    ft.setInt    (ACCESS, ACCESS_RO);
    ft.setInt    (IS_INDEX, 0);
    ft.setInt    (IS_OPTIONAL, 0);
    ft.setString (DESC,   "Logical directory on disk");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME, "defaultInitialFileCount");
    ft.setInt    (TYPE, TYPE_U16);
    ft.setInt    (ACCESS, ACCESS_RO);
    ft.setInt    (IS_INDEX, 0);
    ft.setInt    (IS_OPTIONAL, 0);
    ft.setString (UNIT,   "file");
    ft.setString (DESC,   "Default number of files initially allocated to each journal");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME, "defaultDataFileSize");
    ft.setInt    (TYPE, TYPE_U32);
    ft.setInt    (ACCESS, ACCESS_RO);
    ft.setInt    (IS_INDEX, 0);
    ft.setInt    (IS_OPTIONAL, 0);
    ft.setString (UNIT,   "RdPg");
    ft.setString (DESC,   "Default size of each journal data file");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME, "tplIsInitialized");
    ft.setInt    (TYPE, TYPE_BOOL);
    ft.setInt    (ACCESS, ACCESS_RO);
    ft.setInt    (IS_INDEX, 0);
    ft.setInt    (IS_OPTIONAL, 0);
    ft.setString (DESC,   "Transaction prepared list has been initialized by a transactional prepare");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME, "tplDirectory");
    ft.setInt    (TYPE, TYPE_SSTR);
    ft.setInt    (ACCESS, ACCESS_RO);
    ft.setInt    (IS_INDEX, 0);
    ft.setInt    (IS_OPTIONAL, 0);
    ft.setString (DESC,   "Transaction prepared list directory");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME, "tplWritePageSize");
    ft.setInt    (TYPE, TYPE_U32);
    ft.setInt    (ACCESS, ACCESS_RO);
    ft.setInt    (IS_INDEX, 0);
    ft.setInt    (IS_OPTIONAL, 0);
    ft.setString (UNIT,   "byte");
    ft.setString (DESC,   "Page size in transaction prepared list write-page-cache");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME, "tplWritePages");
    ft.setInt    (TYPE, TYPE_U32);
    ft.setInt    (ACCESS, ACCESS_RO);
    ft.setInt    (IS_INDEX, 0);
    ft.setInt    (IS_OPTIONAL, 0);
    ft.setString (UNIT,   "wpage");
    ft.setString (DESC,   "Number of pages in transaction prepared list write-page-cache");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME, "tplInitialFileCount");
    ft.setInt    (TYPE, TYPE_U16);
    ft.setInt    (ACCESS, ACCESS_RO);
    ft.setInt    (IS_INDEX, 0);
    ft.setInt    (IS_OPTIONAL, 0);
    ft.setString (UNIT,   "file");
    ft.setString (DESC,   "Number of files initially allocated to transaction prepared list journal");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME, "tplDataFileSize");
    ft.setInt    (TYPE, TYPE_U32);
    ft.setInt    (ACCESS, ACCESS_RO);
    ft.setInt    (IS_INDEX, 0);
    ft.setInt    (IS_OPTIONAL, 0);
    ft.setString (UNIT,   "byte");
    ft.setString (DESC,   "Size of each journal data file in transaction prepared list journal");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME, "tplCurrentFileCount");
    ft.setInt    (TYPE, TYPE_U32);
    ft.setInt    (ACCESS, ACCESS_RO);
    ft.setInt    (IS_INDEX, 0);
    ft.setInt    (IS_OPTIONAL, 0);
    ft.setString (UNIT,   "file");
    ft.setString (DESC,   "Number of files currently allocated to transaction prepared list journal");
    buf.put (ft);


    // Statistics
    ft.clear();
    ft.setString (NAME,   "tplTransactionDepth");
    ft.setInt    (TYPE,   TYPE_U32);
    ft.setString (UNIT,   "txn");
    ft.setString (DESC,   "Number of currently enqueued prepared transactions");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME,   "tplTransactionDepthHigh");
    ft.setInt    (TYPE,   TYPE_U32);
    ft.setString (UNIT,   "txn");
    ft.setString (DESC,   "Number of currently enqueued prepared transactions (High)");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME,   "tplTransactionDepthLow");
    ft.setInt    (TYPE,   TYPE_U32);
    ft.setString (UNIT,   "txn");
    ft.setString (DESC,   "Number of currently enqueued prepared transactions (Low)");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME,   "tplTxnPrepares");
    ft.setInt    (TYPE,   TYPE_U64);
    ft.setString (UNIT,   "record");
    ft.setString (DESC,   "Total transaction prepares on transaction prepared list");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME,   "tplTxnCommits");
    ft.setInt    (TYPE,   TYPE_U64);
    ft.setString (UNIT,   "record");
    ft.setString (DESC,   "Total transaction commits on transaction prepared list");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME,   "tplTxnAborts");
    ft.setInt    (TYPE,   TYPE_U64);
    ft.setString (UNIT,   "record");
    ft.setString (DESC,   "Total transaction aborts on transaction prepared list");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME,   "tplOutstandingAIOs");
    ft.setInt    (TYPE,   TYPE_U32);
    ft.setString (UNIT,   "aio_op");
    ft.setString (DESC,   "Number of currently outstanding AIO requests in Async IO system");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME,   "tplOutstandingAIOsHigh");
    ft.setInt    (TYPE,   TYPE_U32);
    ft.setString (UNIT,   "aio_op");
    ft.setString (DESC,   "Number of currently outstanding AIO requests in Async IO system (High)");
    buf.put (ft);

    ft.clear();
    ft.setString (NAME,   "tplOutstandingAIOsLow");
    ft.setInt    (TYPE,   TYPE_U32);
    ft.setString (UNIT,   "aio_op");
    ft.setString (DESC,   "Number of currently outstanding AIO requests in Async IO system (Low)");
    buf.put (ft);


    // Methods

}


void Store::aggregatePerThreadStats(struct PerThreadStats* totals)
{
    totals->tplTxnPrepares = 0;
    totals->tplTxnCommits = 0;
    totals->tplTxnAborts = 0;

    for (int idx = 0; idx < maxThreads; idx++) {
        struct PerThreadStats* threadStats = perThreadStatsArray[idx];
        if (threadStats != 0) {
            totals->tplTxnPrepares += threadStats->tplTxnPrepares;
            totals->tplTxnCommits += threadStats->tplTxnCommits;
            totals->tplTxnAborts += threadStats->tplTxnAborts;

        }
    }
}


void Store::writeProperties (Buffer& buf)
{
    ::qpid::sys::Mutex::ScopedLock mutex(accessLock);
    configChanged = false;

    writeTimestamps (buf);

    brokerRef.encode(buf);
    buf.putShortString(location);
    buf.putShort(defaultInitialFileCount);
    buf.putLong(defaultDataFileSize);
    buf.putOctet(tplIsInitialized?1:0);
    buf.putShortString(tplDirectory);
    buf.putLong(tplWritePageSize);
    buf.putLong(tplWritePages);
    buf.putShort(tplInitialFileCount);
    buf.putLong(tplDataFileSize);
    buf.putLong(tplCurrentFileCount);

}

void Store::writeStatistics (Buffer& buf, bool skipHeaders)
{
    ::qpid::sys::Mutex::ScopedLock mutex(accessLock);
    instChanged = false;


    struct PerThreadStats totals;
    aggregatePerThreadStats(&totals);


    if (!skipHeaders)
        writeTimestamps (buf);
    buf.putLong(tplTransactionDepth);
    buf.putLong(tplTransactionDepthHigh);
    buf.putLong(tplTransactionDepthLow);
    buf.putLongLong(totals.tplTxnPrepares);
    buf.putLongLong(totals.tplTxnCommits);
    buf.putLongLong(totals.tplTxnAborts);
    buf.putLong(tplOutstandingAIOs);
    buf.putLong(tplOutstandingAIOsHigh);
    buf.putLong(tplOutstandingAIOsLow);


    // Maintenance of hi-lo statistics
    tplTransactionDepthHigh = tplTransactionDepth;
    tplTransactionDepthLow  = tplTransactionDepth;
    tplOutstandingAIOsHigh = tplOutstandingAIOs;
    tplOutstandingAIOsLow  = tplOutstandingAIOs;


}

void Store::doMethod (string&, Buffer&, Buffer& outBuf)
{
    Manageable::status_t status = Manageable::STATUS_UNKNOWN_METHOD;
    std::string          text;


    outBuf.putLong(status);
    outBuf.putShortString(Manageable::StatusText(status, text));
}
